@@ -1,4 +1,4 @@
-/* $Id: cy2_saml.c,v 1.9 2012/11/07 16:21:52 manu Exp $ */
+/* $Id: cy2_saml.c,v 1.12 2017/05/24 22:47:15 manu Exp $ */
 
 /*
  * Copyright (c) 2009,2011 Emmanuel Dreyfus
@@ -34,7 +34,7 @@
 #ifdef HAVE_SYS_CDEFS_H
 #include <sys/cdefs.h>
 #ifdef __RCSID
-__RCSID("$Id: cy2_saml.c,v 1.9 2012/11/07 16:21:52 manu Exp $");
+__RCSID("$Id: cy2_saml.c,v 1.12 2017/05/24 22:47:15 manu Exp $");
 #endif
 #endif
 
@@ -58,7 +58,6 @@ __RCSID("$Id: cy2_saml.c,v 1.9 2012/11/07 16:21:52 manu Exp $");
 
 #include "plugin_common.h"
 
-
 typedef struct {
 	char *out;
 	unsigned int len;
@@ -67,12 +66,13 @@ typedef struct {
 static saml_glob_context_t server_glob_context;
 
 void
-saml_log(void *params, int pri, const char *fmt, ...)
+saml_log(const void *utils, int pri, const char *fmt, ...)
 {      
-	sasl_server_params_t *sasl_params;
+	sasl_utils_t *sasl_utils;
+	char msg[4096];
 	va_list ap;
 
-	sasl_params = (sasl_server_params_t *)params;
+	sasl_utils = (sasl_utils_t *)utils;
 	switch (pri) {
 	case LOG_DEBUG:
 		pri = SASL_LOG_DEBUG;
@@ -86,36 +86,37 @@ saml_log(void *params, int pri, const char *fmt, ...)
 	}
 
 	va_start(ap, fmt);
-	sasl_params->utils->log(sasl_params->utils->conn, 
-				pri, fmt, va_arg(ap, char *));
+	vsnprintf(msg, sizeof(msg), fmt, ap);
+	sasl_utils->log(sasl_utils->conn, pri, msg);
 	va_end(ap);
 }      
 
 void
-saml_error(void *params, int pri, const char *fmt, ...)
+saml_error(const void *utils, int pri, const char *fmt, ...)
 {      
-	sasl_server_params_t *sasl_params;
+	sasl_utils_t *sasl_utils;
+	char msg[4096];
 	va_list ap;
 
-	sasl_params = (sasl_server_params_t *)params;
+	sasl_utils = (sasl_utils_t *)utils;
 
 	va_start(ap, fmt);
-	sasl_params->utils->seterror(sasl_params->utils->conn, 
-				     0, fmt, va_arg(ap, char *));
+	vsnprintf(msg, sizeof(msg), fmt, ap);
+	sasl_utils->seterror(sasl_utils->conn, 0, msg);
 	va_end(ap);
 }      
 
 int
-saml_strdup(params, src, dst, len)
-	void *params;
+saml_strdup(utils, src, dst, len)
+	const void *utils;
 	const char *src;
 	char **dst;
 	int *len;
 {
-	sasl_server_params_t *sasl_params;
+	sasl_utils_t *sasl_utils;
 
-	sasl_params = (sasl_server_params_t *)params;
-	return _plug_strdup(sasl_params->utils, src, dst, len);
+	sasl_utils = (sasl_utils_t *)utils;
+	return _plug_strdup(sasl_utils, src, dst, len);
 }
 
 int
@@ -258,7 +259,7 @@ saml_server_mech_step(conn_context, params, clientin, clientinlen,
 	 * Validate SAML assertion, retreive authid
 	 */
         flags = (gctx->flags & SGC_COMPRESSED_ASSERTION) ? MAYBE_COMPRESS : 0;
-	if ((error = saml_check_all_assertions(ctx, params, 
+	if ((error = saml_check_all_assertions(ctx, params->utils, 
 	    &userid, saml_msg, flags)) != SASL_OK)
 		goto out;
 
@@ -346,8 +347,6 @@ saml_server_mech_free(glob_context, utils)
 		gctx->lasso_server = NULL;
 	}
 
-	lasso_shutdown();
-
 	/* 
 	 * Do not free (saml_glob_context_t *)glob_context, it is static!
 	 */
@@ -389,6 +388,7 @@ sasl_server_plug_init(utils, maxvers, outvers, pluglist, plugcount)
 	const char *flag;
 	char propname[1024];
 	int propnum = 0;
+	static int lasso_is_initialized = 0;
 
 	if (maxvers < SASL_SERVER_PLUG_VERSION) {
 		utils->seterror(utils->conn, 0, "SAML version mismatch");
@@ -399,9 +399,11 @@ sasl_server_plug_init(utils, maxvers, outvers, pluglist, plugcount)
 	*pluglist = &saml_server_plugin;
 	*plugcount = 1;
 
-	if (lasso_init() != 0) {
+	if (!lasso_is_initialized && lasso_init() != 0) {
 		utils->seterror(utils->conn, 0, "lasso_init() failed");
 		return SASL_FAIL;
+	} else {
+		lasso_is_initialized = 1;
 	}
 
 	gctx = (saml_glob_context_t *)saml_server_plugin.glob_context;
